*{
  margin: 0;
  padding: 0;
  box-sizing: border-box;
  list-style: none;
  text-decoration: none;
}

html, body {
    height: 100%;
    width:100%;
    text-decoration: none; 
}

#center {
  text-align: center;
}
#options_menu {
  padding: 15pt;
  margin-bottom: 20pt;
  font-variant: small-caps;
  line-height: 2; 
  text-align: center;
}
#grande_galerie{
  color: #F7AE70;
}
#remarks {
  font-style: italic;
  font-variant: normal;
  text-align: center;
	color: grey;
  /* color: #F7AE70; */
  border:2px solid #CCC;
  border-radius: 5px;
	margin-top: 20px;
	padding: 5px;
  font-size: small;
	width: 400px;
	margin-left: auto;
	margin-right: auto;
}

sup {
  color: grey;
}

#additional {
  font-style: italic;
  text-align: center;
  color: #F7AE70;
}
#carte_items {
  text-align: center;
  padding: 15pt;
  font-variant: small-caps;
  line-height: 2; 
}
#carte_item {
  padding: 15pt;
  font-variant: small-caps;
}
img {
  max-width: 20px;
  height: auto;
}
img.logo {
	height: 80px;
  max-width: none;
}
h1, 

h1, h2{
  font-variant: small-caps;
}
h1 {
  text-align: right;
  border-bottom:5px solid #CCC;
  padding-bottom:3px;
  margin-bottom: 15px;
}
h2 {
text-align: center;}

table {
  max-width: 800px;
  margin-top: 1em;
  margin-bottom: 1em;
  margin-left: auto;
  margin-right: auto;
}

table+h1 {
  border-top: none
}

tr:hover {
  background-color: #eee 
}

tr td,
tr th {
  padding: 0.2em 0.7em;
  font-size: small;
}

tr.header {
  border-top: 1px solid #222;
  border-bottom: 1px solid #222;
  font-weight: 700
}



/* tr.odd { */
/*   background-color: #eee */
/* } */

/* tr.even { */
/*   background-color: #ccc */
/* } */

tbody:last-child {
  border-bottom: 1px solid #222
}

dt {
  font-weight: 700
}

dt:after {
  font-weight: normal;
  content: ":"
}

dd {
  margin-bottom: 10px
}
nav{
  overflow: auto;
  white-space: nowrap;
}
label.logo{
  color: #f2f2f2;
  font-size: 33px;
  line-height: 80px;
  padding: 0 140px;
  font-weight: 600;
  font-family: 'Poppins', sans-serif;
}
nav ul{
  /* float: right; */
  /* margin-right: 60px; */
}
nav ul li{
  display: inline-block;
  line-height: 80px;
  margin: 0 2px;
}
nav ul li a{
  color: #f2f2f2;
  font-weight: 500;
  font-size: 20px;
  padding: 7px 13px;
  border-radius: 3px;
  /* text-transform: uppercase; */
  font-family: 'Poppins', sans-serif;
}
a.active,a:hover{
  background: #F7AE70;
  transition: .5s;
}
.checkbtn{
  font-size: 30px;
  color: white;
  float: right;
  line-height: 80px;
  margin-right: 40px;
  cursor: pointer;
  display: none;
}
#check{
  display: none;
}
@media (max-width: 952px){
  label.logo{
    font-size: 27px;
    padding-left: 25px;
  }
  nav ul li a{
    font-size: 16px;
  }
}

.navbar {
  position: fixed;
  top: 0px;
  width: 100%;
  background: #7B7773;
  height: 80px;

}
.main {
	position: absolute;
	top: 100px;
	bottom: 0px;
	left: 0px;
	right: 0px;
	padding: 20px;
	overflow-y: auto;
	/* z-index: -1; */
}

.main_index {
	position: absolute;
	top: 100px;
	bottom: 0px;
	left: 0px;
	right: 0px;
	padding: 20px;
	overflow-y: auto;
	/* z-index: -1; */
}

.main_index h1 {
  text-align: center;
}

/*=============================================================================
header 
https://www.w3schools.com/howto/howto_css_responsive_header.aspground and some padding 
=============================================================================*/

  /* Style the header links */
  .header a {
  display: inline-block;
  /* line-height: 80px; */
  margin: 0 2px;
  color: #a2a2a2;
  font-weight: 500;
  font-size: 20px;
  /* padding: 7px 13px; */
  border-radius: 3px;
  /* text-transform: uppercase; */
  font-family: 'Poppins', sans-serif;
	float: left;
  }

  /* Style the logo link (notice that we set the same value of line-height and font-size to prevent the header to increase when the font gets bigger */
  .header a.logo {
    font-size: 25px;
    font-weight: bold;
  }

  /* Change the background color on mouse-over */
  .header a:hover {
    background-color: #ddd;
    color: black;
  }

  /* Style the active/current link*/
  .header a.active {
    background-color: dodgerblue;
    color: white;
  }

  /* Float the link section to the right */
  .header-right {
    float: right;
  }

.flex-container {
  display: flex;
  flex-wrap: wrap;
justify-content: center;
  <!-- background-color: DodgerBlue; -->
}

.flex-container > div {
  background-color: #f1f1f1;
  width: 300px;
	height: 160px;
  margin: 10px;
  text-align: center;
  /* line-height: 75px; */
  font-size: 30px;
  padding: 10px;
	border-radius: 25px;
	border: 2px solid #F7AE70;
	position: relative;
}

.center {
  margin: 0;
  position: absolute;
  top: 50%;
  left: 50%;
  -ms-transform: translate(-50%, -50%);
  transform: translate(-50%, -50%);
}
